pub mod md5 {
    use md5::Digest;

    /// 计算输入内容的MD5哈希值，返回十六进制字符串。
    ///
    /// - `input` 输入内容。
    pub fn hash<T: AsRef<[u8]>>(input: T) -> String {
        let mut hasher = md5::Md5::new();
        hasher.update(input);
        let result = hasher.finalize();
        crate::serialize::to_hex(result.as_slice())
    }

    /// 计算输入内容的MD5哈希值，返回字节向量。
    ///
    /// - `input` 输入内容。
    pub fn hash_hex<T: AsRef<[u8]>>(input: T) -> Vec<u8> {
        let mut hasher = md5::Md5::new();
        hasher.update(input);
        let result = hasher.finalize();
        result.to_vec()
    }
}

pub mod sha1 {
    use sha1::Digest;

    /// 计算输入内容的SHA1哈希值，返回十六进制字符串。
    ///
    /// - `input` 输入内容。
    pub fn hash<T: AsRef<[u8]>>(input: T) -> String {
        let mut hasher = sha1::Sha1::new();
        hasher.update(input);
        let result = hasher.finalize();
        crate::serialize::to_hex(result.as_slice())
    }

    /// 计算输入内容的SHA1哈希值，返回字节向量。
    ///
    /// - `input` 输入内容。
    pub fn hash_hex<T: AsRef<[u8]>>(input: T) -> Vec<u8> {
        let mut hasher = sha1::Sha1::new();
        hasher.update(input);
        let result = hasher.finalize();
        result.to_vec()
    }
}

pub mod sha512 {

    /// 计算输入内容的SHA512哈希值，返回十六进制字符串。
    ///
    /// - `input` 输入内容。
    pub fn hash<T: AsRef<[u8]>>(input: T) -> String {
        let mut hasher = hmac_sha512::Hash::new();
        hasher.update(input);
        let result = hasher.finalize();
        crate::serialize::to_hex(result.as_slice())
    }

    /// 计算输入内容的SHA512哈希值，返回字节向量。
    ///
    /// - `input` 输入内容。
    pub fn hash_hex<T: AsRef<[u8]>>(input: T) -> Vec<u8> {
        let mut hasher = hmac_sha512::Hash::new();
        hasher.update(input);
        let result = hasher.finalize();
        result.to_vec()
    }
}

pub mod image_hash {
    pub enum Precision {
        Low = 2,
        Medium = 8,
        High = 18,
        Detailed = 32,
    }

    /// 计算输入图片的区块感知哈希值，返回十六进制字符串。
    ///
    /// - `input` 输入图片。
    /// - `precision` 感知精度，感知精度越高，越能区分图片的细节，但是计算时间也越长。
    pub fn hash_image<T: image::GenericImage<Pixel = image::Rgb<u8>>>(
        input: &T,
        precision: Precision,
    ) -> String {
        match precision {
            Precision::Low => blockhash::blockhash16(input).to_string(),
            Precision::Medium => blockhash::blockhash64(input).to_string(),
            Precision::High => blockhash::blockhash144(input).to_string(),
            Precision::Detailed => blockhash::blockhash256(input).to_string(),
        }
    }
}
